// $Id$

#include <AppKit/AppKit.h>

@class NSImage;

@interface UITabActionCell:NSActionCell
{
@private
    float ascender, descender, tabHeight, tabBezierImageWidth;

}

// Methods Overridden From the Superclass
- initTextCell:(NSString *)aString;
- (void)setFont:(NSFont *)fontObj;

- (void)drawInteriorWithFrame:(NSRect)cellFrame inView:(NSView *)controlView;
- (void)highlight:(BOOL)flag withFrame:(NSRect)cellFrame inView:(NSView *)aView;
- (NSSize)cellSizeForBounds:(NSRect)aRect;
- (BOOL)trackMouse:(NSEvent *)theEvent inRect:(NSRect)cellFrame ofView:(NSView *)controlView untilMouseUp:(BOOL)_untilMouseUp;
- (BOOL)startTrackingAt:(NSPoint)startPoint inView:(NSView *)aView;

// Other Instance Methods
- (void)finishInitializing;

// Accessor Methods
- (BOOL)isSelected;
@end