/* Graphics.h
 * Common header of graphic objects
 *
 * Copyright (C) 1996-2002 by vhf interservice GmbH
 * Author:   Georg Fleischmann
 *
 * created:  1996-01-29
 * modified: 2002-07-19
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the vhf Public License as
 * published by vhf interservice GmbH. Among other things, the
 * License requires that the copyright notices and this notice
 * be preserved on all copies.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the vhf Public License for more details.
 *
 * You should have received a copy of the vhf Public License along
 * with this program; see the file LICENSE. If not, write to vhf.
 *
 * vhf interservice GmbH, Im Marxle 3, 72119 Altingen, Germany
 * eMail: info@vhf.de
 * http://www.vhf.de
 */

#ifndef VHF_H_GRAPHICS
#define VHF_H_GRAPHICS

#include "GraphicObjects.subproj/VGraphic.h"
#include "GraphicObjects.subproj/VMark.h"
#include "GraphicObjects.subproj/VWeb.h"
#include "GraphicObjects.subproj/VCurve.h"
#include "GraphicObjects.subproj/VArc.h"
#include "GraphicObjects.subproj/VThread.h"
#include "GraphicObjects.subproj/VLine.h"
#include "GraphicObjects.subproj/VLine3D.h"
#include "GraphicObjects.subproj/VPolyLine.h"
#include "GraphicObjects.subproj/VGroup.h"
#include "GraphicObjects.subproj/VPath.h"
#include "GraphicObjects.subproj/VText.h"
#include "GraphicObjects.subproj/VTextPath.h"
#include "GraphicObjects.subproj/VCrosshairs.h"
#include "GraphicObjects.subproj/VRectangle.h"
#include "GraphicObjects.subproj/ClipRectangle.h"	// old
#include "GraphicObjects.subproj/VImage.h"
#include "GraphicObjects.subproj/VSinking.h"

#endif // VHF_H_GRAPHICS
